#!/usr/bin/env python
import os
import os.path
import xmlrpclib
import pwd
import grp
import sys
import json
import subprocess
import hashlib
import shutil
import lliurex_gva
import gettext
import re
import lliurex.net

gettext.textdomain('lliurex-gva')
_ = gettext.gettext

class mount_gva:

	def __init__(self):
		self.mount_source_name = '.MOUNT_SOURCES'
		self.mount_source_rel_path = os.path.join('.',self.mount_source_name)
		self.domain_group_name = 'usuarios^del^dominio'
		self.mount_L_default = 'windows_home_directory'
		self.user_password = None
		self.windows = lliurex_gva.LliurexGVA(self)

	def check_user_in_domain(self,gid_group_name):
		'''check user in administrador_del_dominio'''
		users_groups = os.getgroups()
		if gid_group_name in users_groups:
			return True
		else:
			return False

	def get_windows_home_directory(self,user_name):
		command = subprocess.Popen(['/opt/pbis/bin/find-user-by-name',user_name],stdout=subprocess.PIPE,stderr=subprocess.PIPE)
		stdout, stderr = command.communicate()
		user_SID = None
		result = None
		for out in stdout.split('\n'):
			if out.startswith('SID'):
				user_SID = out.split(':')[1].strip()
		stderr, stdout, command = None, None, None
		if user_SID != None:
			command = subprocess.Popen(['/opt/pbis/bin/find-objects','--by-sid',user_SID],stdout=subprocess.PIPE,stderr=subprocess.PIPE)
			stdout, stderr = command.communicate()
			
			for out in stdout.split('\n'):
				if out.startswith('Windows home directory'):
					result = out.split(':')[1].strip()
		return result.replace("\\","/")

	def check_windows_home_is_updated(self,path_to_check,windows_home_path_to_check,windows_home_directory):
			#check if needed update remote_file
		if os.path.exists(path_to_check):
			f = open(path_to_check,'r')
			json_mount_list = json.load(f)
			f.close()
		else:
			json_mount_list = {}
		if not json_mount_list.has_key(u'MOUNT_SOURCES'):
			# Create basic struct
			json_mount_list[u'MOUNT_SOURCES'] = {'function':None,'packages':['ClientVariables'],'volatile':False,'description':'N4D mount variable',u'value':{}}

		if not json_mount_list[u'MOUNT_SOURCES'].has_key(u'value'):
			# Create list mount sources
			json_mount_list[u'MOUNT_SOURCES'][u'value'] = {}

		update_file = False
		windows_home_path_to_check=self.user_home+"/"+windows_home_path_to_check
		for orig,mount_object in json_mount_list[u'MOUNT_SOURCES'][u'value'].items():
			if mount_object.has_key(u'windowshome') and mount_object[u'windowshome']:
				#found windows_home_directory on MOUNT_SOURCE remote file
				if orig.strip() != unicode(windows_home_directory.strip()):
					json_mount_list[u'MOUNT_SOURCES'][u'value'].pop(orig)
					json_mount_list[u'MOUNT_SOURCES'][u'value'][windows_home_directory] = {'dst':windows_home_path_to_check,'force_perms':True,'fstype':'cifs','windowshome':True}
					update_file = True
				break
		else:
			#not found windowshome
			json_mount_list[u'MOUNT_SOURCES'][u'value'][windows_home_directory] = {'dst':windows_home_path_to_check,'force_perms':True,'fstype':'cifs','windowshome':True}
			update_file = True

		if update_file :
			f = open(path_to_check,'w')
			data=unicode(json.dumps(json_mount_list,indent=4,encoding="utf-8",ensure_ascii=False)).encode("utf-8")
			f.write(data)
			f.close()
	
	def parse_url(self,source):
		try:
			rule="//\S+/"
			#print source
			x=re.match(rule,source)
			#print x
			hst=x.group().strip("/")

			if not lliurex.net.is_valid_ip(hst):
				ret=lliurex.net.get_ip_from_host(hst)
				if ret!=None:
					source=source.replace(hst,ret)
		except:
			pass
		return source

	def run(self):
		try:
			gid_domain_group = grp.getgrnam(self.domain_group_name).gr_gid
		except Exception as e:
			print e
			gid_domain_group = None

		try:
			user_uid = os.getuid()
			user_name = pwd.getpwuid(user_uid).pw_name
			user_home = pwd.getpwuid(user_uid).pw_dir
			self.user_home=user_home
			local_source_path = os.path.join(user_home,self.mount_source_rel_path)
		except Exception as e:
			print e
			sys.exit(0)

		if not self.check_user_in_domain(gid_domain_group):
			sys.exit(0)


		windows_home_directory = self.get_windows_home_directory(user_name)

		n4d = xmlrpclib.ServerProxy('https://localhost:9779')
		mounted_devices = n4d.mount_list('','MountManager')
		#options = 'username='+ user_name+',uid=' +str( user_uid) + ',gid='+ str(gid_domain_group) +',noperm,_netdev'
		options = 'username='+ user_name+',noperm,_netdev'
		

		#try mount L
		if not isinstance(mounted_devices , dict):
			mounted_devices = {}
		if not windows_home_directory in mounted_devices.keys():
			message = _("Windows home has been detected, but it is not mounted. Input your password to mount it")
			while True:
				self.user_password = None
				self.windows.launch(message)
				self.windows.window.hide()
				if self.user_password == None:
					sys.exit(0)
				output = n4d.restricted_mount((user_name,self.user_password),'MountManager',user_name, windows_home_directory,os.path.join(user_home,self.mount_L_default),'cifs',options + ',password=' + self.user_password)
				if output:
					mounted_devices[windows_home_directory] = {}
					mounted_devices[windows_home_directory]['dst'] = self.mount_L_default
					break
				else:
					message = _("Error: Windows home could not be mounted. Do you want to try again?")
									
				sys.exit(1)


		L_dst = mounted_devices[windows_home_directory]['dst']
		remote_source_path = os.path.join(L_dst,self.mount_source_name)

		if os.path.exists(remote_source_path):
			# Remote file exists
			if os.path.exists(local_source_path):
				local_md5sum = hashlib.md5(local_source_path)
				remote_md5sum = hashlib.md5(remote_source_path)
				if local_md5sum.hexdigest() == remote_md5sum.hexdigest():
					sys.exit(0)
			else:
				if not os.path.exists( os.path.dirname(local_source_path)):
					os.makedirs(os.path.dirname(local_source_path))

			self.check_windows_home_is_updated(remote_source_path,L_dst,windows_home_directory)
			shutil.copy(remote_source_path,local_source_path)

			f = open(local_source_path,'r')
			json_mount_list = json.load(f)
			#json_mount_list['MOUNT_SOURCE']['value']{orig:{'dst':'','force_perms':'','fstype':''}}
			if windows_home_directory in json_mount_list[u'MOUNT_SOURCES'][u'value'].keys():
				json_mount_list[u'MOUNT_SOURCES'][u'value'].pop(unicode(windows_home_directory))
			for dest,obj in json_mount_list[u'MOUNT_SOURCES'][u'value'].items():
				dest = self.parse_url(dest)
				if dest in mounted_devices:
					continue
				if self.user_password == None:
					message = _("Net resources have been detected, but they are not mounted. Input your password to mount them")
					self.windows.launch(message)
				n4d.restricted_mount((user_name,self.user_password),'MountManager',user_name, str(dest), obj[u'dst'],'cifs',options + ',password=' + self.user_password)

		else:
			# Remote file not exists
			if not os.path.exists(local_source_path):
				# local and remote MOUNT_SOURCE not exist
				self.check_windows_home_is_updated(local_source_path,L_dst,windows_home_directory)
			shutil.copy(local_source_path,L_dst)
			sys.exit(0)

if __name__ == '__main__':
	gva = mount_gva()
	gva.run()
