#!/usr/bin/env python
# -*- coding: utf-8 -*

import json
import os.path
import codecs
import shutil
from gi.repository import Gtk, Gdk, GObject, GLib

import signal
signal.signal(signal.SIGINT, signal.SIG_DFL)

import gettext
_=gettext.gettext
gettext.textdomain('lliurex-gva')

class LliurexGVAEditor:

	
	
	def __init__(self):
		
		self.mnt_var={}
		self.add_new_mode=False
		
		self.load_json()
		self.start_gui()
		
	#def __init__
	
	def start_gui(self):
		
		builder=Gtk.Builder()
		builder.set_translation_domain("lliurex-gva")
		if os.path.exists("/srv/svn/pandora/lliurex-gva/trunk/install-files/usr/share/lliurex-gva/rsrc/lliurex-gva-editor.glade"):
			builder.add_from_file("/srv/svn/pandora/lliurex-gva/trunk/install-files/usr/share/lliurex-gva/rsrc/lliurex-gva-editor.glade")
		else:
			builder.add_from_file("/usr/share/lliurex-gva/rsrc/lliurex-gva-editor.glade")
		
		self.window=builder.get_object("main_window")
		self.window.connect("delete_event",self.close_window)
		self.window.show()
		self.treeview=builder.get_object("treeview")
		self.store=Gtk.ListStore(str,str)
		self.treeview.set_model(self.store)
		self.treeview.connect("cursor-changed",self.treeview_clicked)
		src_rend=Gtk.CellRendererText()
		src_column=Gtk.TreeViewColumn(_("Source"))
		src_column.pack_start(src_rend,True)
		src_column.add_attribute(src_rend,"markup",0)
		dest_column=Gtk.TreeViewColumn(_("Mount point"))
		dest_rend=Gtk.CellRendererText()
		dest_column.pack_start(dest_rend,True)
		dest_column.add_attribute(dest_rend,"markup",1)
		self.treeview.append_column(src_column)
		self.treeview.append_column(dest_column)
		self.save_button=builder.get_object("save_button")
		self.save_button.connect("clicked",self.save_clicked)
		self.add_button=builder.get_object("add_button")
		self.add_button.connect("clicked",self.add_clicked)
		self.edit_button=builder.get_object("edit_button")
		self.edit_button.connect("clicked",self.edit_clicked)
		self.delete_button=builder.get_object("delete_button")
		self.delete_button.connect("clicked",self.delete_clicked)
		self.msg_label=builder.get_object("msg_label")
		
		self.edit_dialog=builder.get_object("edit_window")
		self.edit_dialog.connect("delete_event",self.ed_close_window)
		self.ed_apply_button=builder.get_object("ed_apply_button")
		self.ed_apply_button.connect("clicked",self.ed_apply_clicked)
		self.ed_cancel_button=builder.get_object("ed_cancel_button")
		self.ed_cancel_button.connect("clicked",self.ed_cancel_clicked)
		self.ed_src_entry=builder.get_object("src_entry")
		self.ed_dst_chooser=builder.get_object("dst_filechooserbutton")
		
		
		self.close_dialog=builder.get_object("exit_window")
		self.close_dialog.connect("delete_event",self.cd_close)
		self.cd_close=builder.get_object("exit_close_button")
		self.cd_close.connect("clicked",self.cd_close_clicked)
		self.cd_save=builder.get_object("exit_save_button")
		self.cd_save.connect("clicked",self.cd_save_clicked)
		self.cd_cancel=builder.get_object("exit_cancel_button")
		self.cd_cancel.connect("clicked",self.cd_cancel_clicked)
		
		self.populate_treeview()
		Gtk.main()
		
	#def start_gui
	
	def cd_close(self,widget,data):
		
		widget.hide()
		return -1
	
	def cd_close_clicked(self,widget):
		
		Gtk.main_quit()
		
	#def cd_close_clicked
	
	def cd_save_clicked(self,widget):
		
		self.save_values()
		self.close_dialog.hide()
		
	#def cd_save_clicked
	
	def cd_cancel_clicked(self,widget):
		
		self.close_dialog.hide()
		
	#def cd_cancel_clicked
	
	def check_values(self):
		
		if len(self.store)!=len(self.mnt_var["MOUNT_SOURCES"]["value"]):
			
			return False
		
		iter=self.store.get_iter_first()
		
		while iter!=None:
		
			src,dst=self.store.get_value(iter,0).decode("utf-8"),self.store.get_value(iter,1).decode("utf-8")
			if src in self.mnt_var["MOUNT_SOURCES"]["value"]:
				if dst != self.mnt_var["MOUNT_SOURCES"]["value"][src]["dst"]:
					return False
			else:
				
				return False
			
			iter=self.store.iter_next(iter)
			
		return True
		
	#def check_values
	
	def save_values(self):

		try:

			iter=self.store.get_iter_first()
			
			
			tmp={}
			tmp["MOUNT_SOURCES"]={}
			tmp["MOUNT_SOURCES"]["function"]=""
			tmp["MOUNT_SOURCES"]["packages"]=["lliurex-gva"]
			tmp["MOUNT_SOURCES"]["volatile"]=False
			tmp["MOUNT_SOURCES"]["description"]="lliurex-gva mount variable"
			tmp["MOUNT_SOURCES"]["value"]={}
			
			
			iter=self.store.get_iter_first()
			while iter!=None:
		
				src,dst=self.store.get_value(iter,0).decode("utf-8"),self.store.get_value(iter,1).decode("utf-8")

				tmp["MOUNT_SOURCES"]["value"][src]={}
				tmp["MOUNT_SOURCES"]["value"][src]["dst"]=dst
				tmp["MOUNT_SOURCES"]["value"][src]["fstype"]="cifs"
				tmp["MOUNT_SOURCES"]["value"][src]["force_perms"]=True
				
				iter=self.store.iter_next(iter)

			
			self.mnt_var=tmp
			data=json.dumps(self.mnt_var,indent=4,ensure_ascii=False)
			f=codecs.open(os.path.expanduser("~/.MOUNT_SOURCES.tmp"),"w",encoding="utf-8")
			f.write(data)
			f.close()
			
			shutil.move(os.path.expanduser("~/.MOUNT_SOURCES.tmp"),os.path.expanduser("~/.MOUNT_SOURCES"))
			self.msg_label.set_markup("<b>"+_("Mount file saved")+"</b>")
			
			return True
			
		except Exception as e:
			print e
			self.msg_label.set_markup("<span foreground='red'>"+_("Error saving file")+ " [" + str(e) + "]</span>")
			return False
		
	#def save_values
	
	def ed_close_window(self,widget,data):
		
		
		widget.hide()
		return -1
		
		
	#def ed_close_window
	
	
	def ed_apply_clicked(self,widget):

		src=self.ed_src_entry.get_text()
		dst=self.ed_dst_chooser.get_current_folder()
		
		if not self.add_new_mode:
			self.store.set_value(self.current_iter,0,src)
			self.store.set_value(self.current_iter,1,dst)
		else:
			self.store.append((src,dst))
			
		self.add_new_mode=False
		self.edit_dialog.hide()
		
	#def ed_apply_clicked
	
	def ed_cancel_clicked(self,widget):
		
		self.edit_dialog.hide()
		
	#def ed_cancel_clicked
	
	def treeview_clicked(self,widget):
		
		selection=self.treeview.get_selection()
		if selection!=None:
			model,iter=selection.get_selected()
			if iter!=None:
				self.edit_button.set_sensitive(True)
				self.delete_button.set_sensitive(True)
	
	def populate_treeview(self):
		
		try:
			for src in self.mnt_var["MOUNT_SOURCES"]["value"]:
				self.store.append((src,self.mnt_var["MOUNT_SOURCES"]["value"][src]["dst"]))
							
		except Exception as e:
			print e
		
	#def populate
	
	def save_clicked(self,widget):
		
		self.save_values()

		
	#def save_clicked
	
	def edit_clicked(self,widget):

		selection=self.treeview.get_selection()
		model,iter=selection.get_selected()
		if iter!=None:
			
			src=self.store.get_value(iter,0)
			dst=self.store.get_value(iter,1)
			
			self.ed_src_entry.set_text(src)
			self.ed_dst_chooser.set_current_folder(dst)
			self.current_iter=iter
			self.edit_dialog.show()
			

			
			
			
		
	#def edit_clicked
	
	def delete_clicked(self,widget):
		
		selection=self.treeview.get_selection()
		model,iter=selection.get_selected()
		if iter!=None:
			
			src=self.store.get_value(iter,0).decode("utf-8")
			self.store.remove(iter)
			if len(self.store)==0:
				self.edit_button.set_sensitive(False)
				self.delete_button.set_sensitive(False)
				
			
		
		
		
	#def delete_clicked
	
	def add_clicked(self,widget):
		
		self.add_new_mode=True
		self.ed_src_entry.set_text("")
		self.ed_dst_chooser.set_current_folder(os.path.expanduser("~/"))
		self.edit_dialog.show()
		
		
	#def add_clicked

	def close_window(self,widget,date):
		
		if not self.check_values():
			
			self.close_dialog.show()
			return -1
		
		Gtk.main_quit()
		
	#def close_window
	
	def load_json(self, file_path=None):

		self.mnt_var={}
		
		if file_path==None:
			#find mount file here
			file_path=os.path.expanduser("~/.MOUNT_SOURCES")
		
		try:
			f=codecs.open(file_path,"r",encoding="utf-8")
			data=json.load(f)
			f.close()
			self.mnt_var=data
			return True
				
		except Exception as e:
			print(str(e))
			return False
				
		
		
	#def load_json
	
	
if __name__=="__main__":
	
	gui=LliurexGVAEditor()